%  Copyright 2013 Kwang In Kim, Max-Planck-Institut for Informatics
%  Please refer to license.txt for sanctioned usage.
%
%  This code shows a demo of classification based on curvature-aware regularization 
%  Kwang In Kim, James Tompkin, and Christian Theobalt, 
%  "Curvature-aware regularization on riemannian submanifolds", ICCV 2013.
%
%
%  Please note that this code requires a data file containing the USPS database
%  'USPSData.mat' which can be downloaded from http://gvv.mpi-inf.mpg.de/projects/CurvatureAwareRegularization

%  There are 4 parameters to be tuned for regression:
%  1. 'Param.KNSize' specifies the size of nearest neighbors for each data point. 
%     This parameter affects the estimation of local tangent space and the estimation local Hessian.
%  2. 'Param.TanDim' specifies the dimensionality of the manifold
%  3. 'Param.SigmaSq' is the parameter for the weight function in building
%  the graph Laplacian.
%  4. 'Param.Lambda' is the regularization parameter.

% In general, we recommend tuning the parameters using cross-validation based on given labeled data points.
% To facilitate the demo here, we provide a set of pre-optimized parameters.

NumLabels = 100;

load('USPSData.mat', 'Data');

[NumDataPoints,~] = size(Data.x);

%%% parameters
    Param.TanDim    = 3;
    Param.KNSize    = 10;
    Param.SigmaSq   = 2;
    Param.Lambda    = 10/NumDataPoints;
%%% parameters

[NNIdx] = GetKNN(Data.x, Param.KNSize);

%%% selecting labels
    LabelIdx = randperm(NumDataPoints);
    LabelIdx = LabelIdx(1:NumLabels);
%%% selecting labels

%%% preparing label data strucutre
    Labels.y = zeros(size(Data.y));
    Labels.y(LabelIdx) = Data.y(LabelIdx);
    Labels.LFlag = zeros(size(Labels.y));
    Labels.LFlag(LabelIdx) = 1;
%%% preparing label data strucutre

%%% building curvature-aware graph Laplacian
    [L] = ConstructCANormalLap(Data.x, NNIdx, Param.SigmaSq, Param.TanDim);
%%% building curvature-aware graph Laplacian
%%% solving the system
    [f,L] = SemiSupLearn(Labels, L, Param.Lambda);
%%% solving the system

ClsError = length(find(sign(f)~=Data.y))/NumDataPoints;
display(sprintf('Classification error=%f',ClsError*100));
display('done.');
