%  Copyright 2013 Kwang In Kim, Max-Planck-Institut for Informatics
%  Please refer to license.txt for sanctioned usage.
%
function [L] = ConstructCAW(Nodes, NNIdx, SigmaSq, TanDim)

[NumNodes,~] = size(Nodes);

BlSpMatSize = 300000;
MaxNumNN = size(NNIdx,2);
NumNonzero = NumNodes*MaxNumNN;
BlSpMatNum = ceil(NumNonzero/BlSpMatSize);
BlSpL = cell(BlSpMatNum,1);
BlSpCtVal = cell(BlSpMatNum,1);

SpMatIdx = 1;
SpEleCounter = 1;
SpRowIdx = zeros(BlSpMatSize,1);
SpColIdx = zeros(BlSpMatSize,1);
LVal = zeros(BlSpMatSize,1);
CtVal = zeros(BlSpMatSize,1);
for i=1:NumNodes
    G = Nodes(NNIdx(i,:),:);
    
    [NumNeighbors,~] = size(G);
    CtBase = Nodes(i,:);
    [NG,TS] = GetTGCoordG(CtBase, G);
    [NBase,TS] = GetTGCoordGGivenTS(CtBase,TS);
    BaseIndexInG = find(NNIdx(i,:)==i,1);
    
    [LocalL] = ComputeCWGKernel(NG, NBase, BaseIndexInG, SigmaSq, TanDim);

    GRIdx = i;
    GCIdx = NNIdx(i,1:NumNeighbors);
    SpRowIdx(SpEleCounter:SpEleCounter+NumNeighbors-1) = GRIdx;
    SpColIdx(SpEleCounter:SpEleCounter+NumNeighbors-1) = GCIdx;
    LVal(SpEleCounter:SpEleCounter+NumNeighbors-1) = LocalL(1:NumNeighbors);
    CtVal(SpEleCounter:SpEleCounter+NumNeighbors-1) = CtVal(SpEleCounter:SpEleCounter+NumNeighbors-1)+1;
    SpEleCounter = SpEleCounter+NumNeighbors;

    if SpEleCounter > BlSpMatSize-NumNeighbors-1
        NonZeroIdx = length(find(SpRowIdx>0));
        SpRowIdx = SpRowIdx(1:NonZeroIdx);
        SpColIdx = SpColIdx(1:NonZeroIdx);
        LVal = LVal(1:NonZeroIdx);
        CtVal = CtVal(1:NonZeroIdx);
        BlSpL{SpMatIdx} = sparse(SpRowIdx,SpColIdx,LVal,NumNodes,NumNodes);
        BlSpCtVal{SpMatIdx} = sparse(SpRowIdx,SpColIdx,CtVal,NumNodes,NumNodes);
        
        SpMatIdx = SpMatIdx+1;
        SpEleCounter = 1;
        SpRowIdx = zeros(BlSpMatSize,1);
        SpColIdx = zeros(BlSpMatSize,1);
        LVal = zeros(BlSpMatSize,1);
        CtVal = zeros(BlSpMatSize,1);
    end
end

if SpEleCounter > 1
    NonZeroIdx = length(find(SpRowIdx>0));
    SpRowIdx = SpRowIdx(1:NonZeroIdx);
    SpColIdx = SpColIdx(1:NonZeroIdx);
    LVal = LVal(1:NonZeroIdx);
    CtVal = CtVal(1:NonZeroIdx);
    BlSpL{SpMatIdx} = sparse(SpRowIdx,SpColIdx,LVal,NumNodes,NumNodes);
    BlSpCtVal{SpMatIdx} = sparse(SpRowIdx,SpColIdx,CtVal,NumNodes,NumNodes);
        
    SpMatIdx = SpMatIdx+1;
end

L = sparse([],[],[],NumNodes,NumNodes,0);
CtValMat = sparse([],[],[],NumNodes,NumNodes,0);
for i=1:SpMatIdx-1
    L = L+BlSpL{i};
    CtValMat = CtValMat+BlSpCtVal{i};
end
%%% symmetrize nn structure
    L = (L'+L)/2;
    CtIdx = find(CtValMat>1);
    L(CtIdx) = L(CtIdx)./CtValMat(CtIdx);
    L = (L'+L)/2;
%%% symmetrize nn structure
